==> gnome-subtitles-0.1.tar.gz: gnome-subtitles-0.1/sublib-0.4/src/SubLib/Application/SubtitleFactory.cs <==
/*
 * This file is part of SubLib, a library for managing movie subtitles.
 * Copyright (C) 2005-2006 Pedro Castro
 *
 * SubLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SubLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

using System;
using System.Text;

namespace SubLib {

/// <summary>Represents the main mechanism for creating new <see cref="Subtitles" />.</summary>
/// <remarks>A <see cref="SubtitleFactory" /> is no longer needed after the subtitles have been created.</remarks>
public class SubtitleFactory {
	private IncompleteSubtitleCollection incompleteSubtitles = null;
	
	private bool includeIncompleteSubtitles = false;
	
	private Encoding encoding = null;
	private Encoding fallbackEncoding = Encoding.GetEncoding(1252);
	
	private SubtitleType subtitleType = SubtitleType.Unknown;
	
	
	/// <summary>The incomplete subtitles that were found when opening a file.</summary>
	/// <remarks>This is only used when <see cref="IncludeIncompleteSubtitles" /> is set.</remarks>
	public IncompleteSubtitleCollection IncompleteSubtitles {
		 get { return incompleteSubtitles; }
	}
	
	/// <summary>Whether to enable the library to print messages to the console.</summary>
	/// <remarks>Messages will be shown along with the main methods of <see cref="SubtitleFactory" />
	/// and <see cref="Subtitles" />. The default value is false.</remarks>
	public bool Verbose {
		get { return VerboseConsole.Verbose; }
		set { VerboseConsole.Verbose = value; }
	}
	
	/// <summary>Whether to detect and store incomplete subtitles found upon open.</summary>
	/// <remarks>The default value is false.</remarks>
	public bool IncludeIncompleteSubtitles {
		get { return includeIncompleteSubtitles; }
		set { includeIncompleteSubtitles = value; }
	}
	
	/// <summary>The encoding to be used upon open.</summary>
	/// <remarks>When set to null, encoding auto-detection is used. The default value is null (use auto-detection).</remarks>
	public Encoding Encoding {
		get { return encoding; }
		set { encoding = value; }
	}
	
	/// <summary>The encoding to fallback to when using encoding auto-detection.</summary>
	/// <remarks>When using encoding auto-detection, this encoding will be used if no encoding could be auto-detected.
	/// Defaults to Windows-1252.</remarks>
	public Encoding FallbackEncoding {
		get { return fallbackEncoding; }
		set { fallbackEncoding = value; }	
	}
	
	/// <summary>The type of the subtitle being opened.</summary>
	/// <remarks>When set to <see cref="SubtitleType.Unknown" />, subtitle type auto-detection is used.
	/// The default value is <see cref="SubtitleType.Unknown" /> (auto-detection).</remarks>
	public SubtitleType SubtitleType {
		get { return subtitleType; }
		set { subtitleType = value; }
	}
	
	/// <summary>Creates new empty <see cref="Subtitles" />.</summary>
	/// <remarks>Use <see cref="New(string)">New(string)</see> in case the absolute file path is known.
	/// Use <see cref="NewWithName" /> if the file name is known.</remarks>
	/// <returns>The newly created subtitles.</returns>
	public Subtitles New () {
		SubtitleCollection collection = new SubtitleCollection();
		SubtitleProperties properties = new SubtitleProperties();
		return new Subtitles(collection, properties);	
	}
	
	/// <summary>Creates new empty <see cref="Subtitles" /> and sets the absolute file path.</summary>
	/// <remarks>Use <see cref="New()" /> in case the file path isn't available or intended to be used.
	/// Use <see cref="NewWithName" /> if only the file name is known.</remarks>
	/// <param name="fileName">The absolute path where the subtitles will be saved.</param>
	/// <returns>The newly created subtitles.</returns>
	public Subtitles New (string fileName) {
		Subtitles subtitles = New();
		subtitles.Properties.FilePath = fileName;
		return subtitles;
	}
	
	/// <summary>Creates new empty <see cref="Subtitles" /> and sets the file name.</summary>
	/// <remarks>Use <see cref="New()" /> in case the file path isn't available or intended to be used.
	/// Use <see cref="New(string)" /> in case the absolute file path is known.</remarks>
	/// <param name="name">The name of the subtitles's file.</param>
	/// <returns>The newly created subtitles.</returns>
	public Subtitles NewWithName (string name) {
		Subtitles subtitles = New();
		subtitles.Properties.FileName = name;
		return subtitles;	
	}
	
	/// <summary>Creates <see cref="Subtitles" /> by opening the file with the specified name.</summary>
	/// <param name="fileName">The name of the file to be written. This is preferred to be an absolute path.
	/// In case it isn't, the file will be written on the user's current working dir.</param>
	/// <returns>The opened subtitles.</returns>
	public Subtitles Open (string fileName){
		Encoding fileEncoding = null;
		string subtitleText = ReadSubtitleText(fileName, out fileEncoding);
		SubtitleFormat subtitleFormat = GetSubtitleFormat(subtitleText);
		
		SubtitleParser subtitleParser = new SubtitleParser(includeIncompleteSubtitles);
		SubtitleProperties properties = null;
		SubtitleCollection collection = subtitleParser.Parse(subtitleText, subtitleFormat, 
			out properties, out incompleteSubtitles);
			
		Subtitles subtitles = new Subtitles(collection, properties);
		subtitles.SetProperties(subtitleFormat, fileName, fileEncoding);
		CompleteTimingsAfterParsing(subtitles);
		
		VerboseConsole.WriteLine("[*] opened " + fileName + " with encoding " + fileEncoding + " and format " + subtitleFormat.Name);
		return subtitles;
	}
	
	
	/* Private members */

	private string ReadSubtitleText (string fileName, out Encoding fileEncoding) {
		string subtitleText = String.Empty;
		if (encoding == null)
			subtitleText = FileInputOutput.ReadFile(fileName, out fileEncoding, fallbackEncoding, true);
		else {
			subtitleText = FileInputOutput.ReadFile(fileName, encoding, false);
			fileEncoding = encoding;
		}
		return subtitleText;
	}
	
	private SubtitleFormat GetSubtitleFormat (string subtitleText) {
		SubtitleFormat subtitleFormat = null;
		if (subtitleType == SubtitleType.Unknown)
			subtitleFormat = BuiltInSubtitleFormats.Detect(subtitleText);	
		else
			subtitleFormat = BuiltInSubtitleFormats.GetFormat(subtitleType);

		VerboseConsole.WriteLine("Using " + (subtitleType==SubtitleType.Unknown?"detected":"specified")
				+ " subtitle format: " + subtitleFormat);
				
		return subtitleFormat;
	}
	
	private void CompleteTimingsAfterParsing(Subtitles subtitles){
		float originalFrameRate = subtitles.Properties.OriginalFrameRate;
		subtitles.Properties.SetCurrentFrameRate(originalFrameRate);

		if (subtitles.Properties.TimingMode == TimingMode.Times)
			subtitles.UpdateFramesFromTimes(originalFrameRate);
		else
			subtitles.UpdateTimesFromFrames(originalFrameRate);
	}

}

}

==> gnome-subtitles-0.4.tar.gz: gnome-subtitles-0.4/sublib-0.6/src/SubLib/Application/SubtitleFactory.cs <==
/*
 * This file is part of SubLib, a library for managing movie subtitles.
 * Copyright (C) 2005-2007 Pedro Castro
 *
 * SubLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SubLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

using System;
using System.Text;
using System.Text.RegularExpressions;

namespace SubLib {

/// <summary>Represents the main mechanism for creating new <see cref="Subtitles" />.</summary>
/// <remarks>A <see cref="SubtitleFactory" /> is no longer needed after the subtitles have been created.</remarks>
public class SubtitleFactory {
	private IncompleteSubtitleCollection incompleteSubtitles = null;
	private FileProperties fileProperties = null;
	
	private bool includeIncompleteSubtitles = false;
	
	private Encoding encoding = null; //The encoding to be used to open a file 
	private Encoding fallbackEncoding = Encoding.GetEncoding(1252); //The encoding to fall back to when no encoding is detected
	
	private SubtitleType subtitleType = SubtitleType.Unknown;
	
	
	/// <summary>The incomplete subtitles that were found when opening a file.</summary>
	/// <remarks>This is only used when <see cref="IncludeIncompleteSubtitles" /> is set.</remarks>
	public IncompleteSubtitleCollection IncompleteSubtitles {
		 get { return incompleteSubtitles; }
	}
	
	/// <summary>The properties of an opened file, after opening.</summary>
	public FileProperties FileProperties {
		 get { return fileProperties; }
	}
	
	/// <summary>Whether to enable the library to print messages to the console.</summary>
	/// <remarks>Messages will be shown along with the main methods of <see cref="SubtitleFactory" />
	/// and <see cref="Subtitles" />. The default value is false.</remarks>
	public bool Verbose {
		get { return VerboseConsole.Verbose; }
		set { VerboseConsole.Verbose = value; }
	}
	
	/// <summary>Whether to detect and store incomplete subtitles found upon open.</summary>
	/// <remarks>The default value is false.</remarks>
	public bool IncludeIncompleteSubtitles {
		get { return includeIncompleteSubtitles; }
		set { includeIncompleteSubtitles = value; }
	}
	
	/// <summary>The encoding to be used upon open.</summary>
	/// <remarks>When set to null, encoding auto-detection is used. The default value is null (use auto-detection).</remarks>
	public Encoding Encoding {
		get { return encoding; }
		set { encoding = value; }
	}
	
	/// <summary>The encoding to fallback to when using encoding auto-detection.</summary>
	/// <remarks>When using encoding auto-detection, this encoding will be used if no encoding could be auto-detected.
	/// Defaults to Windows-1252.</remarks>
	public Encoding FallbackEncoding {
		get { return fallbackEncoding; }
		set { fallbackEncoding = value; }	
	}
	
	/// <summary>The type of the subtitle being opened.</summary>
	/// <remarks>When set to <see cref="SubtitleType.Unknown" />, subtitle type auto-detection is used.
	/// The default value is <see cref="SubtitleType.Unknown" /> (auto-detection).</remarks>
	public SubtitleType SubtitleType {
		get { return subtitleType; }
		set { subtitleType = value; }
	}
	
	/// <summary>Creates new empty <see cref="Subtitles" />.</summary>
	/// <returns>The newly created subtitles.</returns>
	public Subtitles New () {
		SubtitleCollection collection = new SubtitleCollection();
		SubtitleProperties properties = new SubtitleProperties();
		return new Subtitles(collection, properties);	
	}
	
	/// <summary>Creates <see cref="Subtitles" /> by opening the file at the specified path.</summary>
	/// <remarks>The properties of the opened file are accessible with <see cref="FileProperties" />, after opening.</remarks>
	/// <returns>The opened subtitles.</returns>
	public Subtitles Open (string path){
		Encoding fileEncoding = null;
		string text = ReadSubtitleText(path, out fileEncoding);
		if (IsTextEmpty(text))
			return EmptySubtitles(path);
		else
			return ParsedSubtitles(path, fileEncoding, text);
	}
	
	/* Private members */
		
	private Subtitles ParsedSubtitles (string path, Encoding fileEncoding, string text) {
		SubtitleFormat format = GetSubtitleFormat(text);
		
		SubtitleCollection collection = null;
		SubtitleParser subtitleParser = new SubtitleParser(includeIncompleteSubtitles);
		ParsingProperties parsingProperties = subtitleParser.Parse(text, format, out collection, out incompleteSubtitles);
		
		SubtitleProperties subtitleProperties = new SubtitleProperties(parsingProperties);
		collection.SetPropertiesForAll(subtitleProperties);
		
		Subtitles subtitles = new Subtitles(collection, subtitleProperties);
		CompleteTimingsAfterParsing(subtitles, parsingProperties);
		
		fileProperties = new FileProperties(path, fileEncoding, format.Type , parsingProperties.TimingMode);

		VerboseConsole.WriteLine("[*] opened " + path + " with encoding " + fileEncoding + " and format " + format.Name);
		return subtitles;
	}
	
	private Subtitles EmptySubtitles (string path) {
		Subtitles subtitles = New();
		fileProperties = new FileProperties(path, Encoding.UTF8, SubtitleType.Unknown, TimingMode.Times);
		return subtitles;
	}
	
	private bool IsTextEmpty (string text) {
		Regex regex = new Regex(@"\s*");
		Match match = regex.Match(text);
		return (match.Length == text.Length);
	}

	private string ReadSubtitleText (string fileName, out Encoding fileEncoding) {
		string subtitleText = String.Empty;
		if (encoding == null)
			subtitleText = FileInputOutput.ReadFile(fileName, out fileEncoding, fallbackEncoding, true);
		else {
			subtitleText = FileInputOutput.ReadFile(fileName, encoding, false);
			fileEncoding = encoding;
		}
		return subtitleText;
	}
	
	private SubtitleFormat GetSubtitleFormat (string subtitleText) {
		SubtitleFormat subtitleFormat = null;
		if (subtitleType == SubtitleType.Unknown)
			subtitleFormat = BuiltInSubtitleFormats.Detect(subtitleText);	
		else
			subtitleFormat = BuiltInSubtitleFormats.GetFormat(subtitleType);

		VerboseConsole.WriteLine("Using " + (subtitleType==SubtitleType.Unknown?"detected":"specified")
				+ " subtitle format: " + subtitleFormat);
				
		return subtitleFormat;
	}
	
	private void CompleteTimingsAfterParsing(Subtitles subtitles, ParsingProperties parsingProperties){
		float originalFrameRate = subtitles.Properties.OriginalFrameRate;
		subtitles.Properties.SetCurrentFrameRate(originalFrameRate);

		if (parsingProperties.TimingMode == TimingMode.Times)
			subtitles.UpdateFramesFromTimes(originalFrameRate);
		else
			subtitles.UpdateTimesFromFrames(originalFrameRate);
	}

}

}
